/*-------------------------------------------------------------------------
 *
 * pg_largeobject_metadata_d.h
 *    Macro definitions for pg_largeobject_metadata
 *
 * Portions Copyright (c) 1996-2019, PostgreSQL Global Development Group
 * Portions Copyright (c) 1994, Regents of the University of California
 *
 * NOTES
 *  ******************************
 *  *** DO NOT EDIT THIS FILE! ***
 *  ******************************
 *
 *  It has been GENERATED by src/backend/catalog/genbki.pl
 *
 *-------------------------------------------------------------------------
 */
#ifndef PG_LARGEOBJECT_METADATA_D_H
#define PG_LARGEOBJECT_METADATA_D_H

#define LargeObjectMetadataRelationId 2995

#define Anum_pg_largeobject_metadata_oid 1
#define Anum_pg_largeobject_metadata_lomowner 2
#define Anum_pg_largeobject_metadata_lomacl 3

#define Natts_pg_largeobject_metadata 3


#endif							/* PG_LARGEOBJECT_METADATA_D_H */
